/* InsetsUIResource.java
   Copyright (C) 2002, 2003 Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */


package javax.swing.plaf;

import java.awt.Insets;
import java.io.Serializable;


/**
 * An <code>Insets</code> that is marked as <code>UIResource</code>,
 * which indicates that it has been installed by a pluggable
 * LookAndFeel. Such insets are replaced when the LookAndFeel changes.
 *
 * @author Andrew Selkirk (aselkirk@sympatico.ca)
 * @author Sascha Brawer (brawer@dandelis.ch)
 */
public class InsetsUIResource extends Insets
  implements Cloneable, UIResource, Serializable
{
  /**
   * Determined using the <code>serialver</code> tool
   * of Apple/Sun JDK 1.3.1 on MacOS X 10.1.5.
   */
  static final long serialVersionUID = 5622110143266315421L;


  /**
   * Constructs a new <code>InsetsUIResource</code> given the
   * inset at each edge.
   *
   * @param top the inset at the top, in pixels.
   * @param left the inset at the left, in pixels.
   * @param bottom the inset at the bottom, in pixels.
   * @param right the inset at the right, in pixels.
   */
  public InsetsUIResource(int top, int left, int bottom, int right)
  {
    super(top, left, bottom, right);
  }
}
