/* NamingContextHolder.java --
   Copyright (C) 2005 Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */


package org.omg.CosNaming;

import org.omg.CORBA.TypeCode;
import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA.portable.OutputStream;
import org.omg.CORBA.portable.Streamable;

/**
 * A holder for the naming context.
 *
 * @author Audrius Meskauskas, Lithuania (AudriusA@Bioinformatics.org)
 */
public final class NamingContextHolder
  implements Streamable
{
  /**
   * A stored value.
   */
  public NamingContext value;

  /**
   * Create the unitialised naming context holder.
   */
  public NamingContextHolder()
  {
  }

  /**
   * Create the naming context holder, intialised to the given value.
   */
  public NamingContextHolder(NamingContext initialValue)
  {
    value = initialValue;
  }

  /**
   * Read the naming context from the given CDR input stream.
   */
  public void _read(InputStream in)
  {
    value = NamingContextHelper.read(in);
  }

  /**
   * Get the typecode of the naming context.
   */
  public TypeCode _type()
  {
    return NamingContextHelper.type();
  }

  /**
   * Write the naming context to the given CDR output stream.
   */
  public void _write(OutputStream out)
  {
    NamingContextHelper.write(out, value);
  }
}
